unit XColorComboBox;

{
  TXColorComboBox 1.0 (1999-10-04)
  ================================
  This component enables you to select one of 41 predefined standard and
  system colors used in Delphi Object Inspector.

  Freeware.

  Copyright  1999 Roman Stedronsky, Roman.Stedronsky@seznam.cz.

  All rights reserved. You may use this software in an application
  without fee or royalty, provided this copyright notice remains intact.

  properties:

    SelectedColor: TColor	read/write current color selection	

}

{$S-,W-,R-}
{$C PRELOAD}

interface

uses
  Windows, Classes, Graphics, Controls, StdCtrls;

type
  TXColorComboBox = class(TCustomComboBox)
  private
    function GetSelectedColor: TColor;
    procedure SetSelectedColor(Value: TColor);
  protected
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property SelectedColor: TColor read GetSelectedColor write SetSelectedColor;
    // publish propeties from TXustomComboBox
    property Anchors;
    property BiDiMode;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property MaxLength;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Extra', [TXColorComboBox]);
end;

resourcestring
  rsBlack = 'Black';
  rsMaroon = 'Maroon';
  rsGreen = 'Green';
  rsOlive = 'Olive';
  rsNavy = 'Navy';
  rsPurple = 'Purple';
  rsTeal = 'Teal';
  rsGray = 'Gray';
  rsSilver = 'Silver';
  rsRed = 'Red';
  rsLime = 'Lime';
  rsYellow = 'Yellow';
  rsBlue = 'Blue';
  rsFuchsia = 'Fuchsia';
  rsAqua = 'Aqua';
  rsWhite = 'White';
  rsScrollBar = 'ScrollBar';
  rsBackground = 'Background';
  rsActiveCaption = 'Active Caption';
  rsInactiveCaption = 'Inactive Caption';
  rsMenu = 'Menu';
  rsWindow = 'Window';
  rsWindowFrame = 'Window Frame';
  rsMenuText = 'Menu Text';
  rsWindowText = 'Window Text';
  rsCaptionText = 'Caption Text';
  rsActiveBorder = 'Active Border';
  rsInactiveBorder = 'Inactive Border';
  rsAppWorkSpace = 'Application Work Space';
  rsHighlight = 'Highlight';
  rsHighlightText = 'Highlight Text';
  rsBtnFace = 'Button Face';
  rsBtnShadow = 'Button Shadow';
  rsGrayText = 'Gray Text';
  rsBtnText = 'Button Text';
  rsInactiveCaptionText = 'Inactive Caption Text';
  rsBtnHighlight = 'Button Highlight';
  rs3DDkShadow = '3D Dark Shadow';
  rs3DLight = '3D Light';
  rsInfoText = 'Info Text';
  rsInfoBk = 'Info Background';
  rsNone = 'None';

const
  Colors: array[0..41] of TIdentMapEntry = (
    (Value: clBlack; Name: rsBlack),
    (Value: clMaroon; Name: rsMaroon),
    (Value: clGreen; Name: rsGreen),
    (Value: clOlive; Name: rsOlive),
    (Value: clNavy; Name: rsNavy),
    (Value: clPurple; Name: rsPurple),
    (Value: clTeal; Name: rsTeal),
    (Value: clGray; Name: rsGray),
    (Value: clSilver; Name: rsSilver),
    (Value: clRed; Name: rsRed),
    (Value: clLime; Name: rsLime),
    (Value: clYellow; Name: rsYellow),
    (Value: clBlue; Name: rsBlue),
    (Value: clFuchsia; Name: rsFuchsia),
    (Value: clAqua; Name: rsAqua),
    (Value: clWhite; Name: rsWhite),
    (Value: clScrollBar; Name: rsScrollBar),
    (Value: clBackground; Name: rsBackground),
    (Value: clActiveCaption; Name: rsActiveCaption),
    (Value: clInactiveCaption; Name: rsInactiveCaption),
    (Value: clMenu; Name: rsMenu),
    (Value: clWindow; Name: rsWindow),
    (Value: clWindowFrame; Name: rsWindowFrame),
    (Value: clMenuText; Name: rsMenuText),
    (Value: clWindowText; Name: rsWindowText),
    (Value: clCaptionText; Name: rsCaptionText),
    (Value: clActiveBorder; Name: rsActiveBorder),
    (Value: clInactiveBorder; Name: rsInactiveBorder),
    (Value: clAppWorkSpace; Name: rsAppWorkSpace),
    (Value: clHighlight; Name: rsHighlight),
    (Value: clHighlightText; Name: rsHighlightText),
    (Value: clBtnFace; Name: rsBtnFace),
    (Value: clBtnShadow; Name: rsBtnShadow),
    (Value: clGrayText; Name: rsGrayText),
    (Value: clBtnText; Name: rsBtnText),
    (Value: clInactiveCaptionText; Name: rsInactiveCaptionText),
    (Value: clBtnHighlight; Name: rsBtnHighlight),
    (Value: cl3DDkShadow; Name: rs3DDkShadow),
    (Value: cl3DLight; Name: rs3DLight),
    (Value: clInfoText; Name: rsInfoText),
    (Value: clInfoBk; Name: rsInfoBk),
    (Value: clNone; Name: rsNone));

{ TXColorComboBox }

function TXColorComboBox.GetSelectedColor: TColor;
begin
  Result := Colors[ItemIndex].Value;
end;

procedure TXColorComboBox.SetSelectedColor(Value: TColor);
var
  Index: Integer;
begin
  for Index := Low(Colors) to High(Colors) do
    if Colors[Index].Value = Value then
    begin
      ItemIndex := Index;
      Exit;
    end;
end;

procedure TXColorComboBox.CreateWnd;
var
  Index: integer;
begin
  inherited CreateWnd;
  Items.BeginUpdate;
  for Index := 0 to 41 do
    Items.Add(Colors[Index].Name);
  Items.EndUpdate;
  ItemIndex := 0;
end;

procedure TXColorComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Square: TRect;
begin
  TControlCanvas(Canvas).UpdateTextFlags;
  if odSelected in State then
  begin
    Canvas.Brush.Color := clHighlight;
  end;
  Canvas.FillRect(Rect);

  Canvas.Brush.Color := clBlack;
  Square := Rect;
  Square.Left := Square.Left + 1;
  Square.Right := Square.Left + ItemHeight - 2;
  Square.Top := Square.Top + 1;
  Square.Bottom := Square.Bottom - 1;
  Canvas.FrameRect(Square);

  Canvas.Brush.Color := Colors[Index].Value;
  InflateRect(Square, -1, -1);
  Canvas.FillRect(Square);

  if odFocused in State then
  begin
    Canvas.Brush.Color := clHighlightText;
    Canvas.DrawFocusRect(Rect);
  end;

  Inc(Rect.Left, ItemHeight);
  InflateRect(Rect, 0, -1);
  if odSelected in State then
  begin
    Canvas.Pen.Color := clHighlightText;
    Canvas.Brush.Color := clHighlight;
  end
  else
  begin
    Canvas.Pen.Color := Font.Color;
    Canvas.Brush.Color := Color;
  end;
  DrawText(Canvas.Handle, PChar(Items[Index]), Length(Items[Index]), Rect, DT_VCENTER or DT_LEFT or DT_SINGLELINE);
end;

constructor TXColorComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
end;

destructor TXColorComboBox.Destroy;
begin
  inherited Destroy;
end;

end.

